/*
* This file is part of rasdaman community.
*
* Rasdaman community is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* Rasdaman community is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with rasdaman community.  If not, see <http://www.gnu.org/licenses/>.
*
* Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009 Peter Baumann /
rasdaman GmbH.
*
* For more information please see <http://www.rasdaman.org>
* or contact Peter Baumann via <baumann@rasdaman.com>.
*/
#include "collectiontype.hh"
#include "mddtype.hh"

r_Bytes
CollectionType::getMemorySize() const
	{
	return DBNamedObject::getMemorySize() + myMDDType->getMemorySize() + sizeof(MDDType*);
	}

CollectionType::CollectionType(const MDDType* newMDDType)
	:	Type("unnamed collectiontype")
	{
	myMDDType = newMDDType;
	}

CollectionType::CollectionType()
	:	Type("unnamed collectiontype")
	{
	myMDDType = 0;
	}

CollectionType::CollectionType(const char* name)
	:	Type(name)
	{
	myMDDType = 0;
	}


CollectionType::CollectionType(const char* name, const MDDType* newMDDType)
	:	Type(name)
	{
	myMDDType = newMDDType;
	}

CollectionType::CollectionType(const OId& id) throw (r_Error)
	:	Type(id)
	{
	}

CollectionType::CollectionType(const CollectionType& old)
	:	Type(old)
	{
	myMDDType = old.myMDDType;
	}

CollectionType&
CollectionType::operator=(const CollectionType& old)
	{
	Type::operator=(old);
	myMDDType = old.myMDDType;
	return *this; 
	}

CollectionType::~CollectionType()
	{
	}

const MDDType*
CollectionType::getMDDType() const
	{
	return myMDDType;
	}

void
CollectionType::print_status( ostream& s ) const
{
  s << "d_" << getName() << "<";
  myMDDType->print_status(s);
  s << " >";
}

int
CollectionType::compatibleWith(const Type* aType) const
{
  return myMDDType->compatibleWith(aType);
}
